using System;
using System.IO;

namespace protocol2html {
	public class App {
		[STAThread]
		static void Main(string[] args) {
			string[] files=Directory.GetFiles(".\\", "*.xml");
			foreach(string s in files) {
				int index=s.LastIndexOf('.');
				string outfile=s.Substring(0, index)+".html";

				try {
					Console.WriteLine("Processing "+s+"...");
					ConvertToHtml(s, outfile);
				}
				catch(Exception ex) {
					Console.Error.WriteLine("Exception Occured: "+ex.Message);
				}
			}
		}

		// System.Web.HttpUtility.HtmlEncode doesn't encode some binary values.
		static string HtmlEncode(string str) {
			string ret=string.Empty;

			foreach(char c in str) {
				switch(c) {
					case '<': ret+="&lt;"; break;
					case '>': ret+="&gt;"; break;
					case '&': ret+="&amp;"; break;
					default:
						if(c >= 0x20 && c <= 0x7e) ret+=c;
						else ret+="&#"+((int)c).ToString("D3")+";";
						break;
				}
			}

			return ret;
		}

		static void ConvertToHtml(string infile, string outfile) {
			Protocol protocol=Protocol.FromFile(infile);

			StreamWriter sw=new StreamWriter(outfile, false, System.Text.Encoding.ASCII);

			sw.WriteLine("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
			sw.WriteLine("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"");
			sw.WriteLine("\t\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
			sw.WriteLine();
			sw.WriteLine("<html>");
			sw.WriteLine("\t<head>");
			sw.WriteLine("\t\t<title>"+HtmlEncode(protocol.Name)+"</title>");
			sw.WriteLine("\t\t<link rel=\"stylesheet\" type=\"text/css\" href=\"../default.css\" />");
			sw.WriteLine("\t\t<link rel=\"stylesheet\" type=\"text/css\" href=\"protocols.css\" />");
			sw.WriteLine("\t</head>");
			sw.WriteLine("\t<body>");
			sw.WriteLine("\t\t<h1>"+HtmlEncode(protocol.Name)+"</h1>");

			if(protocol.Packets!=null) {
				foreach(Packet p in protocol.Packets) {
					sw.WriteLine("\t\t<div class=\"packet\">");

					sw.WriteLine("\t\t\t<h2>"+HtmlEncode(p.Name)+"</h2>");

					sw.WriteLine("\t\t\t<table summary=\"packet\">");
					sw.WriteLine("\t\t\t\t<tr><td colspan=\"2\"><h3>Query</h3></td></tr>");
					WritePacketInfo(sw, p.Query);

					sw.WriteLine("\t\t\t\t<tr><td colspan=\"2\"><h3>Response</h3></td></tr>");
					WritePacketInfo(sw, p.Response);
					sw.WriteLine("\t\t\t</table>");

					sw.WriteLine("\t\t</div>");
				}
			}

			sw.WriteLine("\t\t<div class=\"bottom\">");
			sw.WriteLine("\t\t\t<a href=\"http://www.int64.org\"><img src=\"../button.png\" alt=\"int64.org - When 4GiB of RAM just isn't enough.\" /></a>");
			sw.WriteLine("\t\t\t<a href=\"http://sourceforge.net/projects/int64\"><img src=\"http://sourceforge.net/sflogo.php?group_id=112954&amp;type=1\" alt=\"SourceForge.net Logo\" /></a>");
			sw.WriteLine("\t\t</div>");
			sw.WriteLine("\t\t<div class=\"license\">");
			sw.WriteLine("\t\t\t<a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/2.0/\"><img alt=\"Creative Commons License\" src=\"http://creativecommons.org/images/public/somerights20.gif\" /></a><br />");
			sw.WriteLine("\t\t\tThis work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/2.0/\">Creative Commons License</a>.");
			sw.WriteLine("\t\t</div>");

			sw.WriteLine("\t</body>");
			sw.WriteLine("</html>");

			sw.Close();
		}

		static int GetColorRange(Range[] ranges, int pos) {
			if(ranges!=null) {
				foreach(Range r in ranges)
					if(pos >= r.Start && pos <= (r.Start+r.Length-1)) return r.Color;
			}
			return -1;
		}

		static bool IsDelimiter(Delimiter[] delimiters, char c) {
			if(delimiters!=null) {
				foreach(Delimiter d in delimiters)
					if(c==d.Value[0]) return true;
			}
			return false;
		}

		static void WritePacketInfo(StreamWriter sw, PacketInfo pi) {
			sw.WriteLine("\t\t\t\t<tr>");
			sw.WriteLine("\t\t\t\t\t<td>");
			sw.WriteLine("\t\t\t\t\t\t<pre class=\"packetdata\">");

			if(pi.Delimiters!=null) {
				for(int i=0; i<pi.Delimiters.Length; i++) {
					switch(pi.Delimiters[i].Value) {
						case "\\\\": pi.Delimiters[i].Value="\\"; break;
						case "\\t": pi.Delimiters[i].Value="\t"; break;
						case "\\r": pi.Delimiters[i].Value="\r"; break;
						case "\\n": pi.Delimiters[i].Value="\n"; break;
					}
				}
			}

			string hex=string.Empty, bin=string.Empty;
			int curcolor=-1;

			for(int i=0; i<pi.Bin.Length;) {
				int j;
				for(j=0; i<pi.Bin.Length && j<16; j++, i++) {
					int color=GetColorRange(pi.Ranges, i);

					if(color!=curcolor) {
						if(curcolor!=-1) {
							hex+="</span>";
							bin+="</span>";
						}
						curcolor=color;

						if(j!=0) hex+=' ';

						if(color!=-1) {
							string s="<span class=\"color_"+color.ToString()+"\">";
							hex+=s;
							bin+=s;
						}
					}
					else if(j!=0) hex+=' ';

					char ch=Convert.ToChar(pi.Bin[i]);

					bool isdelim=false;
					if(color==-1 && IsDelimiter(pi.Delimiters, ch)) {
						isdelim=true;
						hex+="<span class=\"delimiter\">";
						bin+="<span class=\"delimiter\">";
					}

					hex+=HtmlEncode(pi.Bin[i].ToString("X2"));
					switch(ch) {
						case '\t': case '\r': case '\n': case '\0':
							bin+='.';
							break;
						default:
							bin+=HtmlEncode(ch.ToString());
							break;
					}

					if(isdelim) {
						hex+="</span>";
						bin+="</span>";
					}
				}

				if(curcolor!=-1) {
					hex+="</span>";
					bin+="</span>";
				}

				while(j++ < 16) {
					hex+="   ";
				}

				sw.WriteLine(hex+" ;"+bin);

				hex=string.Empty;
				bin=string.Empty;

				curcolor=-1;
			}

			sw.WriteLine("</pre>");

			sw.WriteLine("\t\t\t\t\t</td>");
			sw.WriteLine("\t\t\t\t\t<td>");


			if(pi.Ranges!=null) {
				sw.WriteLine("\t\t\t\t\t\t<div>");
				sw.WriteLine("\t\t\t\t\t\t\t<h4>Key</h4>");

				foreach(Range r in pi.Ranges) {
					sw.WriteLine("\t\t\t\t\t\t\t<span class=\"colortext_{0}\"><span>({1}) {2}</span></span><br />",
						r.Color, HtmlEncode(r.Type), HtmlEncode(r.Description));
				}

				sw.WriteLine("\t\t\t\t\t\t</div>");
			}

			if(pi.Comments!=null && pi.Comments.Length>0) {
				sw.WriteLine("\t\t\t\t\t\t<div>");
				sw.WriteLine("\t\t\t\t\t\t\t<h4>Comments</h4>");
				sw.WriteLine(pi.Comments);
				sw.WriteLine("\t\t\t\t\t\t</div>");
			}
			
			sw.WriteLine("\t\t\t\t\t</td>");
			sw.WriteLine("\t\t\t\t</tr>");
		}
	}
}
